@@ -33,8 +33,9 @@ module Language.C.Syntax.AST (
   CStructTag(..), CStructureUnion(..),  CEnumeration(..),
   -- * Declaration attributes
   CDeclSpec, partitionDeclSpecs,
-  CStorageSpec, CTypeSpec, isSUEDef, CTypeQual, CAttr,
-  CDeclarationSpecifier(..), CStorageSpecifier(..), CTypeSpecifier(..),
+  CStorageSpec, CTypeSpec, isSUEDef, CTypeQual, CFunSpec, CAlignSpec,  CAttr,
+  CFunctionSpecifier(..), CDeclarationSpecifier(..), CStorageSpecifier(..), CTypeSpecifier(..),
+  CAlignmentSpecifier(..),
   CTypeQualifier(..), CAttribute(..),
   -- * Declarators
   CDeclr,CDerivedDeclr,CArrSize,
@@ -56,7 +57,6 @@ module Language.C.Syntax.AST (
   -- * Annoated type class
   Annotated(..)
 ) where
-import Data.List
 import Language.C.Syntax.Constants
 import Language.C.Syntax.Ops
 import Language.C.Data.Ident
@@ -156,7 +156,11 @@ data CDeclaration a
     [(Maybe (CDeclarator a),  -- declarator (may be omitted)
       Maybe (CInitializer a), -- optional initialize
       Maybe (CExpression a))] -- optional size (const expr)
-    a
+    a                         -- annotation
+    | CStaticAssert
+      (CExpression a)         -- assert expression
+      (CStringLiteral a)      -- assert text
+      a                       -- annotation
     deriving (Show, Data,Typeable {-! ,CNode ,Annotated !-})
 
 -- Derive instance is a little bit ugly
@@ -164,6 +168,8 @@ instance Functor CDeclaration where
   fmap f (CDecl specs declarators annot) =
     CDecl (map (fmap f) specs) (map fmap3m declarators) (f annot)
       where fmap3m (a,b,c) = (fmap (fmap f) a, fmap (fmap f) b, fmap (fmap f) c)
+  fmap f (CStaticAssert expression strlit annot) =
+    CStaticAssert (fmap f expression) (fmap f strlit) (f annot)
 
 -- | C declarator (K&R A8.5, C99 6.7.5) and abstract declarator (K&R A8.8, C99 6.7.6)
 --
@@ -381,23 +387,26 @@ data CDeclarationSpecifier a
   = CStorageSpec (CStorageSpecifier a) -- ^ storage-class specifier or typedef
   | CTypeSpec    (CTypeSpecifier a)    -- ^ type name
   | CTypeQual    (CTypeQualifier a)    -- ^ type qualifier
+  | CFunSpec     (CFunctionSpecifier a) -- ^ function specifier
+  | CAlignSpec   (CAlignmentSpecifier a) -- ^ alignment specifier
     deriving (Show, Data,Typeable {-! ,CNode ,Functor, Annotated !-})
 
 
 -- | Separate the declaration specifiers
 --
--- Note that inline isn't actually a type qualifier, but a function specifier.
 -- @__attribute__@ of a declaration qualify declarations or declarators (but not types),
 -- and are therefore separated as well.
 partitionDeclSpecs :: [CDeclarationSpecifier a]
                    -> ( [CStorageSpecifier a], [CAttribute a]
-                      , [CTypeQualifier a], [CTypeSpecifier a], Bool)
-partitionDeclSpecs = foldr deals ([],[],[],[],False) where
-    deals (CTypeQual (CInlineQual _)) (sts,ats,tqs,tss,_) = (sts,ats,tqs,tss,True)
-    deals (CStorageSpec sp) (sts,ats,tqs,tss,inline)  = (sp:sts,ats,tqs,tss,inline)
-    deals (CTypeQual (CAttrQual attr)) (sts,ats,tqs,tss,inline)  = (sts,attr:ats,tqs,tss,inline)
-    deals (CTypeQual tq) (sts,ats,tqs,tss,inline)     = (sts,ats,tq:tqs,tss,inline)
-    deals (CTypeSpec ts) (sts,ats,tqs,tss,inline)     = (sts,ats,tqs,ts:tss,inline)
+                      , [CTypeQualifier a], [CTypeSpecifier a]
+                      , [CFunctionSpecifier a], [CAlignmentSpecifier a])
+partitionDeclSpecs = foldr deals ([],[],[],[],[],[]) where
+    deals (CStorageSpec sp) (sts,ats,tqs,tss,fss,ass)  = (sp:sts,ats,tqs,tss,fss,ass)
+    deals (CTypeQual (CAttrQual attr)) (sts,ats,tqs,tss,fss,ass)  = (sts,attr:ats,tqs,tss,fss,ass)
+    deals (CTypeQual tq) (sts,ats,tqs,tss,fss,ass)     = (sts,ats,tq:tqs,tss,fss,ass)
+    deals (CTypeSpec ts) (sts,ats,tqs,tss,fss,ass)     = (sts,ats,tqs,ts:tss,fss,ass)
+    deals (CFunSpec fs) (sts,ats,tqs,tss,fss,ass)      = (sts,ats,tqs,tss,fs:fss,ass)
+    deals (CAlignSpec as) (sts,ats,tqs,tss,fss,ass)    = (sts,ats,tqs,tss,fss,as:ass)
 
 -- | C storage class specifier (and typedefs) (K&R A8.1, C99 6.7.1)
 type CStorageSpec = CStorageSpecifier NodeInfo
@@ -407,7 +416,7 @@ data CStorageSpecifier a
   | CStatic   a     -- ^ static
   | CExtern   a     -- ^ extern
   | CTypedef  a     -- ^ typedef
-  | CThread   a     -- ^ GNUC thread local storage
+  | CThread   a     -- ^ C11/GNUC thread local storage
     deriving (Show, Eq,Ord,Data,Typeable {-! ,CNode ,Functor ,Annotated !-})
 
 
@@ -430,11 +439,13 @@ data CTypeSpecifier a
   | CUnsigType   a
   | CBoolType    a
   | CComplexType a
+  | CInt128Type  a
   | CSUType      (CStructureUnion a) a      -- ^ Struct or Union specifier
   | CEnumType    (CEnumeration a)    a      -- ^ Enumeration specifier
   | CTypeDef     Ident        a      -- ^ Typedef name
   | CTypeOfExpr  (CExpression a)  a  -- ^ @typeof(expr)@
   | CTypeOfType  (CDeclaration a) a  -- ^ @typeof(type)@
+  | CAtomicType  (CDeclaration a) a  -- ^ @_Atomic(type)@
     deriving (Show, Data,Typeable {-! ,CNode ,Functor ,Annotated !-})
 
 
@@ -444,17 +455,37 @@ isSUEDef (CSUType (CStruct _ _ (Just _) _ _) _) = True
 isSUEDef (CEnumType (CEnum _ (Just _) _ _) _) = True
 isSUEDef _ = False
 
--- | C type qualifiers (K&R A8.2, C99 6.7.3), function specifiers (C99 6.7.4), and attributes.
+-- | C type qualifiers (K&R A8.2, C99 6.7.3) and attributes.
 --
--- @const@, @volatile@ and @restrict@ type qualifiers and @inline@ function specifier.
--- Additionally, @__attribute__@ annotations for declarations and declarators.
+-- @const@, @volatile@ and @restrict@ type qualifiers
+-- Additionally, @__attribute__@ annotations for declarations and declarators, and
+-- function specifiers
 type CTypeQual = CTypeQualifier NodeInfo
 data CTypeQualifier a
   = CConstQual a
   | CVolatQual a
   | CRestrQual a
-  | CInlineQual a
+  | CAtomicQual a
   | CAttrQual  (CAttribute a)
+  | CNullableQual a
+  | CNonnullQual a
+    deriving (Show, Data,Typeable {-! ,CNode ,Functor ,Annotated !-})
+
+-- | C function specifiers (C99 6.7.4)
+--
+-- function specifiers @inline@ and @_Noreturn@
+type CFunSpec = CFunctionSpecifier NodeInfo
+data CFunctionSpecifier a
+  = CInlineQual a
+  | CNoreturnQual a
+    deriving (Show, Data,Typeable {-! ,CNode ,Functor ,Annotated !-})
+
+-- | C alignment specifiers (C99 6.7.5)
+--
+type CAlignSpec = CAlignmentSpecifier NodeInfo
+data CAlignmentSpecifier a
+  = CAlignAsType (CDeclaration a) a  -- ^ @_Alignas(type)@
+  | CAlignAsExpr (CExpression a) a   -- ^ @_Alignas(expr)@
     deriving (Show, Data,Typeable {-! ,CNode ,Functor ,Annotated !-})
 
 
@@ -639,6 +670,7 @@ data CExpression a
   | CCompoundLit (CDeclaration a)
                  (CInitializerList a)    -- type name & initialiser list
                  a                       -- ^ C99 compound literal
+  | CGenericSelection (CExpression a) [(Maybe (CDeclaration a), CExpression a)] a -- ^ C11 generic selection
   | CStatExpr    (CStatement a) a        -- ^ GNU C compound statement as expr
   | CLabAddrExpr Ident a                 -- ^ GNU C address of label
   | CBuiltinExpr (CBuiltinThing a)       -- ^ builtin expressions, see 'CBuiltin'
@@ -673,7 +705,10 @@ instance Functor CExpression where
         fmap _f (CStatExpr a1 a2) = CStatExpr (fmap _f a1) (_f a2)
         fmap _f (CLabAddrExpr a1 a2) = CLabAddrExpr a1 (_f a2)
         fmap _f (CBuiltinExpr a1) = CBuiltinExpr (fmap _f a1)
-
+        fmap _f (CGenericSelection expr list annot) =
+          CGenericSelection (fmap _f expr) (map fmap_helper list) (_f annot)
+          where
+            fmap_helper (ma1, a2) = (fmap (fmap _f) ma1, fmap _f a2)
 
 -- | GNU Builtins, which cannot be typed in C99
 type CBuiltin = CBuiltinThing NodeInfo
@@ -725,39 +760,31 @@ class (Functor ast) => Annotated ast where
 -- Instances generated using derive-2.*
 -- GENERATED START
 
-
-instance (CNode t1) => CNode (CTranslationUnit t1) where
+instance CNode t1 => CNode (CTranslationUnit t1) where
         nodeInfo (CTranslUnit _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CTranslationUnit t1) where
+instance CNode t1 => Pos (CTranslationUnit t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CTranslationUnit where
         fmap _f (CTranslUnit a1 a2)
           = CTranslUnit (fmap (fmap _f) a1) (_f a2)
 
-
 instance Annotated CTranslationUnit where
         annotation (CTranslUnit _ n) = n
         amap f (CTranslUnit a_1 a_2) = CTranslUnit a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CExternalDeclaration t1) where
+instance CNode t1 => CNode (CExternalDeclaration t1) where
         nodeInfo (CDeclExt d) = nodeInfo d
         nodeInfo (CFDefExt d) = nodeInfo d
         nodeInfo (CAsmExt _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CExternalDeclaration t1) where
+instance CNode t1 => Pos (CExternalDeclaration t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CExternalDeclaration where
         fmap _f (CDeclExt a1) = CDeclExt (fmap _f a1)
         fmap _f (CFDefExt a1) = CFDefExt (fmap _f a1)
         fmap _f (CAsmExt a1 a2) = CAsmExt (fmap _f a1) (_f a2)
 
-
 instance Annotated CExternalDeclaration where
         annotation (CDeclExt n) = annotation n
         annotation (CFDefExt n) = annotation n
@@ -766,68 +793,57 @@ instance Annotated CExternalDeclaration where
         amap f (CFDefExt n) = CFDefExt (amap f n)
         amap f (CAsmExt a_1 a_2) = CAsmExt a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CFunctionDef t1) where
+instance CNode t1 => CNode (CFunctionDef t1) where
         nodeInfo (CFunDef _ _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CFunctionDef t1) where
+instance CNode t1 => Pos (CFunctionDef t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CFunctionDef where
         fmap _f (CFunDef a1 a2 a3 a4 a5)
           = CFunDef (fmap (fmap _f) a1) (fmap _f a2) (fmap (fmap _f) a3)
               (fmap _f a4)
               (_f a5)
 
-
 instance Annotated CFunctionDef where
         annotation (CFunDef _ _ _ _ n) = n
         amap f (CFunDef a_1 a_2 a_3 a_4 a_5)
           = CFunDef a_1 a_2 a_3 a_4 (f a_5)
 
-
-instance (CNode t1) => CNode (CDeclaration t1) where
+instance CNode t1 => CNode (CDeclaration t1) where
         nodeInfo (CDecl _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CDeclaration t1) where
+        nodeInfo (CStaticAssert _ _ n) = nodeInfo n
+instance CNode t1 => Pos (CDeclaration t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Annotated CDeclaration where
         annotation (CDecl _ _ n) = n
+        annotation (CStaticAssert _ _ n) = n
         amap f (CDecl a_1 a_2 a_3) = CDecl a_1 a_2 (f a_3)
+        amap f (CStaticAssert a_1 a_2 a_3) = CStaticAssert a_1 a_2 (f a_3)
 
-
-instance (CNode t1) => CNode (CDeclarator t1) where
+instance CNode t1 => CNode (CDeclarator t1) where
         nodeInfo (CDeclr _ _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CDeclarator t1) where
+instance CNode t1 => Pos (CDeclarator t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CDeclarator where
         fmap _f (CDeclr a1 a2 a3 a4 a5)
           = CDeclr a1 (fmap (fmap _f) a2) (fmap (fmap _f) a3)
               (fmap (fmap _f) a4)
               (_f a5)
 
-
 instance Annotated CDeclarator where
         annotation (CDeclr _ _ _ _ n) = n
         amap f (CDeclr a_1 a_2 a_3 a_4 a_5)
           = CDeclr a_1 a_2 a_3 a_4 (f a_5)
 
-
-instance (CNode t1) => CNode (CDerivedDeclarator t1) where
+instance CNode t1 => CNode (CDerivedDeclarator t1) where
         nodeInfo (CPtrDeclr _ n) = nodeInfo n
         nodeInfo (CArrDeclr _ _ n) = nodeInfo n
         nodeInfo (CFunDeclr _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CDerivedDeclarator t1) where
+instance CNode t1 => Pos (CDerivedDeclarator t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Annotated CDerivedDeclarator where
         annotation (CPtrDeclr _ n) = n
         annotation (CArrDeclr _ _ n) = n
@@ -836,13 +852,11 @@ instance Annotated CDerivedDeclarator where
         amap f (CArrDeclr a_1 a_2 a_3) = CArrDeclr a_1 a_2 (f a_3)
         amap f (CFunDeclr a_1 a_2 a_3) = CFunDeclr a_1 a_2 (f a_3)
 
-
 instance Functor CArraySize where
         fmap _ (CNoArrSize a1) = CNoArrSize a1
         fmap _f (CArrSize a1 a2) = CArrSize a1 (fmap _f a2)
 
-
-instance (CNode t1) => CNode (CStatement t1) where
+instance CNode t1 => CNode (CStatement t1) where
         nodeInfo (CLabel _ _ _ n) = nodeInfo n
         nodeInfo (CCase _ _ n) = nodeInfo n
         nodeInfo (CCases _ _ _ n) = nodeInfo n
@@ -859,11 +873,9 @@ instance (CNode t1) => CNode (CStatement t1) where
         nodeInfo (CBreak d) = nodeInfo d
         nodeInfo (CReturn _ n) = nodeInfo n
         nodeInfo (CAsm _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CStatement t1) where
+instance CNode t1 => Pos (CStatement t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Annotated CStatement where
         annotation (CLabel _ _ _ n) = n
         annotation (CCase _ _ n) = n
@@ -898,14 +910,11 @@ instance Annotated CStatement where
         amap f (CReturn a_1 a_2) = CReturn a_1 (f a_2)
         amap f (CAsm a_1 a_2) = CAsm a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CAssemblyStatement t1) where
+instance CNode t1 => CNode (CAssemblyStatement t1) where
         nodeInfo (CAsmStmt _ _ _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CAssemblyStatement t1) where
+instance CNode t1 => Pos (CAssemblyStatement t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CAssemblyStatement where
         fmap _f (CAsmStmt a1 a2 a3 a4 a5 a6)
           = CAsmStmt (fmap (fmap _f) a1) (fmap _f a2) (fmap (fmap _f) a3)
@@ -913,46 +922,37 @@ instance Functor CAssemblyStatement where
               (fmap (fmap _f) a5)
               (_f a6)
 
-
 instance Annotated CAssemblyStatement where
         annotation (CAsmStmt _ _ _ _ _ n) = n
         amap f (CAsmStmt a_1 a_2 a_3 a_4 a_5 a_6)
           = CAsmStmt a_1 a_2 a_3 a_4 a_5 (f a_6)
 
-
-instance (CNode t1) => CNode (CAssemblyOperand t1) where
+instance CNode t1 => CNode (CAssemblyOperand t1) where
         nodeInfo (CAsmOperand _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CAssemblyOperand t1) where
+instance CNode t1 => Pos (CAssemblyOperand t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CAssemblyOperand where
         fmap _f (CAsmOperand a1 a2 a3 a4)
           = CAsmOperand a1 (fmap _f a2) (fmap _f a3) (_f a4)
 
-
 instance Annotated CAssemblyOperand where
         annotation (CAsmOperand _ _ _ n) = n
         amap f (CAsmOperand a_1 a_2 a_3 a_4)
           = CAsmOperand a_1 a_2 a_3 (f a_4)
 
-
-instance (CNode t1) => CNode (CCompoundBlockItem t1) where
+instance CNode t1 => CNode (CCompoundBlockItem t1) where
         nodeInfo (CBlockStmt d) = nodeInfo d
         nodeInfo (CBlockDecl d) = nodeInfo d
         nodeInfo (CNestedFunDef d) = nodeInfo d
-
-instance (CNode t1) => Pos (CCompoundBlockItem t1) where
+instance CNode t1 => Pos (CCompoundBlockItem t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CCompoundBlockItem where
         fmap _f (CBlockStmt a1) = CBlockStmt (fmap _f a1)
         fmap _f (CBlockDecl a1) = CBlockDecl (fmap _f a1)
         fmap _f (CNestedFunDef a1) = CNestedFunDef (fmap _f a1)
 
-
 instance Annotated CCompoundBlockItem where
         annotation (CBlockStmt n) = annotation n
         annotation (CBlockDecl n) = annotation n
@@ -961,43 +961,44 @@ instance Annotated CCompoundBlockItem where
         amap f (CBlockDecl n) = CBlockDecl (amap f n)
         amap f (CNestedFunDef n) = CNestedFunDef (amap f n)
 
-
-instance (CNode t1) => CNode (CDeclarationSpecifier t1) where
+instance CNode t1 => CNode (CDeclarationSpecifier t1) where
         nodeInfo (CStorageSpec d) = nodeInfo d
         nodeInfo (CTypeSpec d) = nodeInfo d
         nodeInfo (CTypeQual d) = nodeInfo d
-
-instance (CNode t1) => Pos (CDeclarationSpecifier t1) where
+        nodeInfo (CFunSpec d) = nodeInfo d
+        nodeInfo (CAlignSpec d) = nodeInfo d
+instance CNode t1 => Pos (CDeclarationSpecifier t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CDeclarationSpecifier where
         fmap _f (CStorageSpec a1) = CStorageSpec (fmap _f a1)
         fmap _f (CTypeSpec a1) = CTypeSpec (fmap _f a1)
         fmap _f (CTypeQual a1) = CTypeQual (fmap _f a1)
-
+        fmap _f (CFunSpec a1) = CFunSpec (fmap _f a1)
+        fmap _f (CAlignSpec a1) = CAlignSpec (fmap _f a1)
 
 instance Annotated CDeclarationSpecifier where
         annotation (CStorageSpec n) = annotation n
         annotation (CTypeSpec n) = annotation n
         annotation (CTypeQual n) = annotation n
+        annotation (CFunSpec n) = annotation n
+        annotation (CAlignSpec n) = annotation n
         amap f (CStorageSpec n) = CStorageSpec (amap f n)
         amap f (CTypeSpec n) = CTypeSpec (amap f n)
         amap f (CTypeQual n) = CTypeQual (amap f n)
+        amap f (CFunSpec n) = CFunSpec (amap f n)
+        amap f (CAlignSpec n) = CAlignSpec (amap f n)
 
-
-instance (CNode t1) => CNode (CStorageSpecifier t1) where
+instance CNode t1 => CNode (CStorageSpecifier t1) where
         nodeInfo (CAuto d) = nodeInfo d
         nodeInfo (CRegister d) = nodeInfo d
         nodeInfo (CStatic d) = nodeInfo d
         nodeInfo (CExtern d) = nodeInfo d
         nodeInfo (CTypedef d) = nodeInfo d
         nodeInfo (CThread d) = nodeInfo d
-
-instance (CNode t1) => Pos (CStorageSpecifier t1) where
+instance CNode t1 => Pos (CStorageSpecifier t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CStorageSpecifier where
         fmap _f (CAuto a1) = CAuto (_f a1)
         fmap _f (CRegister a1) = CRegister (_f a1)
@@ -1006,7 +1007,6 @@ instance Functor CStorageSpecifier where
         fmap _f (CTypedef a1) = CTypedef (_f a1)
         fmap _f (CThread a1) = CThread (_f a1)
 
-
 instance Annotated CStorageSpecifier where
         annotation (CAuto n) = n
         annotation (CRegister n) = n
@@ -1021,8 +1021,7 @@ instance Annotated CStorageSpecifier where
         amap f (CTypedef a_1) = CTypedef (f a_1)
         amap f (CThread a_1) = CThread (f a_1)
 
-
-instance (CNode t1) => CNode (CTypeSpecifier t1) where
+instance CNode t1 => CNode (CTypeSpecifier t1) where
         nodeInfo (CVoidType d) = nodeInfo d
         nodeInfo (CCharType d) = nodeInfo d
         nodeInfo (CShortType d) = nodeInfo d
@@ -1034,16 +1033,16 @@ instance (CNode t1) => CNode (CTypeSpecifier t1) where
         nodeInfo (CUnsigType d) = nodeInfo d
         nodeInfo (CBoolType d) = nodeInfo d
         nodeInfo (CComplexType d) = nodeInfo d
+        nodeInfo (CInt128Type d) = nodeInfo d
         nodeInfo (CSUType _ n) = nodeInfo n
         nodeInfo (CEnumType _ n) = nodeInfo n
         nodeInfo (CTypeDef _ n) = nodeInfo n
         nodeInfo (CTypeOfExpr _ n) = nodeInfo n
         nodeInfo (CTypeOfType _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CTypeSpecifier t1) where
+        nodeInfo (CAtomicType _ n) = nodeInfo n
+instance CNode t1 => Pos (CTypeSpecifier t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CTypeSpecifier where
         fmap _f (CVoidType a1) = CVoidType (_f a1)
         fmap _f (CCharType a1) = CCharType (_f a1)
@@ -1056,12 +1055,13 @@ instance Functor CTypeSpecifier where
         fmap _f (CUnsigType a1) = CUnsigType (_f a1)
         fmap _f (CBoolType a1) = CBoolType (_f a1)
         fmap _f (CComplexType a1) = CComplexType (_f a1)
+        fmap _f (CInt128Type a1) = CInt128Type (_f a1)
         fmap _f (CSUType a1 a2) = CSUType (fmap _f a1) (_f a2)
         fmap _f (CEnumType a1 a2) = CEnumType (fmap _f a1) (_f a2)
         fmap _f (CTypeDef a1 a2) = CTypeDef a1 (_f a2)
         fmap _f (CTypeOfExpr a1 a2) = CTypeOfExpr (fmap _f a1) (_f a2)
         fmap _f (CTypeOfType a1 a2) = CTypeOfType (fmap _f a1) (_f a2)
-
+        fmap _f (CAtomicType a1 a2) = CAtomicType (fmap _f a1) (_f a2)
 
 instance Annotated CTypeSpecifier where
         annotation (CVoidType n) = n
@@ -1075,11 +1075,13 @@ instance Annotated CTypeSpecifier where
         annotation (CUnsigType n) = n
         annotation (CBoolType n) = n
         annotation (CComplexType n) = n
+        annotation (CInt128Type n) = n
         annotation (CSUType _ n) = n
         annotation (CEnumType _ n) = n
         annotation (CTypeDef _ n) = n
         annotation (CTypeOfExpr _ n) = n
         annotation (CTypeOfType _ n) = n
+        annotation (CAtomicType _ n) = n
         amap f (CVoidType a_1) = CVoidType (f a_1)
         amap f (CCharType a_1) = CCharType (f a_1)
         amap f (CShortType a_1) = CShortType (f a_1)
@@ -1091,114 +1093,138 @@ instance Annotated CTypeSpecifier where
         amap f (CUnsigType a_1) = CUnsigType (f a_1)
         amap f (CBoolType a_1) = CBoolType (f a_1)
         amap f (CComplexType a_1) = CComplexType (f a_1)
+        amap f (CInt128Type a_1) = CInt128Type (f a_1)
         amap f (CSUType a_1 a_2) = CSUType a_1 (f a_2)
         amap f (CEnumType a_1 a_2) = CEnumType a_1 (f a_2)
         amap f (CTypeDef a_1 a_2) = CTypeDef a_1 (f a_2)
         amap f (CTypeOfExpr a_1 a_2) = CTypeOfExpr a_1 (f a_2)
         amap f (CTypeOfType a_1 a_2) = CTypeOfType a_1 (f a_2)
+        amap f (CAtomicType a_1 a_2) = CAtomicType a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CTypeQualifier t1) where
+instance CNode t1 => CNode (CTypeQualifier t1) where
         nodeInfo (CConstQual d) = nodeInfo d
         nodeInfo (CVolatQual d) = nodeInfo d
         nodeInfo (CRestrQual d) = nodeInfo d
-        nodeInfo (CInlineQual d) = nodeInfo d
+        nodeInfo (CAtomicQual d) = nodeInfo d
         nodeInfo (CAttrQual d) = nodeInfo d
+        nodeInfo (CNullableQual d) = nodeInfo d
+        nodeInfo (CNonnullQual d) = nodeInfo d
 
-instance (CNode t1) => Pos (CTypeQualifier t1) where
+instance CNode t1 => Pos (CTypeQualifier t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CTypeQualifier where
         fmap _f (CConstQual a1) = CConstQual (_f a1)
         fmap _f (CVolatQual a1) = CVolatQual (_f a1)
         fmap _f (CRestrQual a1) = CRestrQual (_f a1)
-        fmap _f (CInlineQual a1) = CInlineQual (_f a1)
+        fmap _f (CAtomicQual a1) = CAtomicQual (_f a1)
         fmap _f (CAttrQual a1) = CAttrQual (fmap _f a1)
-
+        fmap _f (CNullableQual a1) = CNullableQual (_f a1)
+        fmap _f (CNonnullQual a1) = CNonnullQual (_f a1)
 
 instance Annotated CTypeQualifier where
         annotation (CConstQual n) = n
         annotation (CVolatQual n) = n
         annotation (CRestrQual n) = n
-        annotation (CInlineQual n) = n
+        annotation (CAtomicQual n) = n
         annotation (CAttrQual n) = annotation n
+        annotation (CNullableQual n) = n
+        annotation (CNonnullQual n) = n
         amap f (CConstQual a_1) = CConstQual (f a_1)
         amap f (CVolatQual a_1) = CVolatQual (f a_1)
         amap f (CRestrQual a_1) = CRestrQual (f a_1)
-        amap f (CInlineQual a_1) = CInlineQual (f a_1)
+        amap f (CAtomicQual a_1) = CAtomicQual (f a_1)
         amap f (CAttrQual n) = CAttrQual (amap f n)
+        amap f (CNullableQual a_1) = CNullableQual (f a_1)
+        amap f (CNonnullQual a_1) = CNonnullQual (f a_1)
 
-
-instance (CNode t1) => CNode (CStructureUnion t1) where
-        nodeInfo (CStruct _ _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CStructureUnion t1) where
+instance CNode t1 => CNode (CFunctionSpecifier t1) where
+        nodeInfo (CInlineQual d) = nodeInfo d
+        nodeInfo (CNoreturnQual d) = nodeInfo d
+instance CNode t1 => Pos (CFunctionSpecifier t1) where
         posOf x = posOf (nodeInfo x)
 
+instance Functor CFunctionSpecifier where
+        fmap _f (CInlineQual a1) = CInlineQual (_f a1)
+        fmap _f (CNoreturnQual a1) = CNoreturnQual (_f a1)
+
+instance Annotated CFunctionSpecifier where
+        annotation (CInlineQual n) = n
+        annotation (CNoreturnQual n) = n
+        amap f (CInlineQual a_1) = CInlineQual (f a_1)
+        amap f (CNoreturnQual a_1) = CNoreturnQual (f a_1)
+
+instance CNode t1 => CNode (CAlignmentSpecifier t1) where
+        nodeInfo (CAlignAsType _ n) = nodeInfo n
+        nodeInfo (CAlignAsExpr _ n) = nodeInfo n
+instance CNode t1 => Pos (CAlignmentSpecifier t1) where
+        posOf x = posOf (nodeInfo x)
+
+instance Functor CAlignmentSpecifier where
+        fmap _f (CAlignAsType a1 a2) = CAlignAsType (fmap _f a1) (_f a2)
+        fmap _f (CAlignAsExpr a1 a2) = CAlignAsExpr (fmap _f a1) (_f a2)
+
+instance Annotated CAlignmentSpecifier where
+        annotation (CAlignAsType _ n) = n
+        annotation (CAlignAsExpr _ n) = n
+        amap f (CAlignAsType a_1 a_2) = CAlignAsType a_1 (f a_2)
+        amap f (CAlignAsExpr a_1 a_2) = CAlignAsExpr a_1 (f a_2)
+
+instance CNode t1 => CNode (CStructureUnion t1) where
+        nodeInfo (CStruct _ _ _ _ n) = nodeInfo n
+instance CNode t1 => Pos (CStructureUnion t1) where
+        posOf x = posOf (nodeInfo x)
 
 instance Functor CStructureUnion where
         fmap _f (CStruct a1 a2 a3 a4 a5)
           = CStruct a1 a2 (fmap (fmap (fmap _f)) a3) (fmap (fmap _f) a4)
               (_f a5)
 
-
 instance Annotated CStructureUnion where
         annotation (CStruct _ _ _ _ n) = n
         amap f (CStruct a_1 a_2 a_3 a_4 a_5)
           = CStruct a_1 a_2 a_3 a_4 (f a_5)
 
-
-instance (CNode t1) => CNode (CEnumeration t1) where
+instance CNode t1 => CNode (CEnumeration t1) where
         nodeInfo (CEnum _ _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CEnumeration t1) where
+instance CNode t1 => Pos (CEnumeration t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CEnumeration where
         fmap _f (CEnum a1 a2 a3 a4)
           = CEnum a1 (fmap (fmap (fmap (fmap (fmap _f)))) a2)
               (fmap (fmap _f) a3)
               (_f a4)
 
-
 instance Annotated CEnumeration where
         annotation (CEnum _ _ _ n) = n
         amap f (CEnum a_1 a_2 a_3 a_4) = CEnum a_1 a_2 a_3 (f a_4)
 
-
-instance (CNode t1) => CNode (CInitializer t1) where
+instance CNode t1 => CNode (CInitializer t1) where
         nodeInfo (CInitExpr _ n) = nodeInfo n
         nodeInfo (CInitList _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CInitializer t1) where
+instance CNode t1 => Pos (CInitializer t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Annotated CInitializer where
         annotation (CInitExpr _ n) = n
         annotation (CInitList _ n) = n
         amap f (CInitExpr a_1 a_2) = CInitExpr a_1 (f a_2)
         amap f (CInitList a_1 a_2) = CInitList a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CPartDesignator t1) where
+instance CNode t1 => CNode (CPartDesignator t1) where
         nodeInfo (CArrDesig _ n) = nodeInfo n
         nodeInfo (CMemberDesig _ n) = nodeInfo n
         nodeInfo (CRangeDesig _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CPartDesignator t1) where
+instance CNode t1 => Pos (CPartDesignator t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CPartDesignator where
         fmap _f (CArrDesig a1 a2) = CArrDesig (fmap _f a1) (_f a2)
         fmap _f (CMemberDesig a1 a2) = CMemberDesig a1 (_f a2)
         fmap _f (CRangeDesig a1 a2 a3)
           = CRangeDesig (fmap _f a1) (fmap _f a2) (_f a3)
 
-
 instance Annotated CPartDesignator where
         annotation (CArrDesig _ n) = n
         annotation (CMemberDesig _ n) = n
@@ -1207,24 +1233,19 @@ instance Annotated CPartDesignator where
         amap f (CMemberDesig a_1 a_2) = CMemberDesig a_1 (f a_2)
         amap f (CRangeDesig a_1 a_2 a_3) = CRangeDesig a_1 a_2 (f a_3)
 
-
-instance (CNode t1) => CNode (CAttribute t1) where
+instance CNode t1 => CNode (CAttribute t1) where
         nodeInfo (CAttr _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CAttribute t1) where
+instance CNode t1 => Pos (CAttribute t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CAttribute where
         fmap _f (CAttr a1 a2 a3) = CAttr a1 (fmap (fmap _f) a2) (_f a3)
 
-
 instance Annotated CAttribute where
         annotation (CAttr _ _ n) = n
         amap f (CAttr a_1 a_2 a_3) = CAttr a_1 a_2 (f a_3)
 
-
-instance (CNode t1) => CNode (CExpression t1) where
+instance CNode t1 => CNode (CExpression t1) where
         nodeInfo (CComma _ n) = nodeInfo n
         nodeInfo (CAssign _ _ _ n) = nodeInfo n
         nodeInfo (CCond _ _ _ n) = nodeInfo n
@@ -1243,14 +1264,13 @@ instance (CNode t1) => CNode (CExpression t1) where
         nodeInfo (CVar _ n) = nodeInfo n
         nodeInfo (CConst d) = nodeInfo d
         nodeInfo (CCompoundLit _ _ n) = nodeInfo n
+        nodeInfo (CGenericSelection _ _ n) = nodeInfo n
         nodeInfo (CStatExpr _ n) = nodeInfo n
         nodeInfo (CLabAddrExpr _ n) = nodeInfo n
         nodeInfo (CBuiltinExpr d) = nodeInfo d
-
-instance (CNode t1) => Pos (CExpression t1) where
+instance CNode t1 => Pos (CExpression t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Annotated CExpression where
         annotation (CComma _ n) = n
         annotation (CAssign _ _ _ n) = n
@@ -1270,6 +1290,7 @@ instance Annotated CExpression where
         annotation (CVar _ n) = n
         annotation (CConst n) = annotation n
         annotation (CCompoundLit _ _ n) = n
+        annotation (CGenericSelection _ _ n) = n
         annotation (CStatExpr _ n) = n
         annotation (CLabAddrExpr _ n) = n
         annotation (CBuiltinExpr n) = annotation n
@@ -1291,20 +1312,19 @@ instance Annotated CExpression where
         amap f (CVar a_1 a_2) = CVar a_1 (f a_2)
         amap f (CConst n) = CConst (amap f n)
         amap f (CCompoundLit a_1 a_2 a_3) = CCompoundLit a_1 a_2 (f a_3)
+        amap f (CGenericSelection a_1 a_2 a_3)
+          = CGenericSelection a_1 a_2 (f a_3)
         amap f (CStatExpr a_1 a_2) = CStatExpr a_1 (f a_2)
         amap f (CLabAddrExpr a_1 a_2) = CLabAddrExpr a_1 (f a_2)
         amap f (CBuiltinExpr n) = CBuiltinExpr (amap f n)
 
-
-instance (CNode t1) => CNode (CBuiltinThing t1) where
+instance CNode t1 => CNode (CBuiltinThing t1) where
         nodeInfo (CBuiltinVaArg _ _ n) = nodeInfo n
         nodeInfo (CBuiltinOffsetOf _ _ n) = nodeInfo n
         nodeInfo (CBuiltinTypesCompatible _ _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CBuiltinThing t1) where
+instance CNode t1 => Pos (CBuiltinThing t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CBuiltinThing where
         fmap _f (CBuiltinVaArg a1 a2 a3)
           = CBuiltinVaArg (fmap _f a1) (fmap _f a2) (_f a3)
@@ -1313,7 +1333,6 @@ instance Functor CBuiltinThing where
         fmap _f (CBuiltinTypesCompatible a1 a2 a3)
           = CBuiltinTypesCompatible (fmap _f a1) (fmap _f a2) (_f a3)
 
-
 instance Annotated CBuiltinThing where
         annotation (CBuiltinVaArg _ _ n) = n
         annotation (CBuiltinOffsetOf _ _ n) = n
@@ -1324,24 +1343,20 @@ instance Annotated CBuiltinThing where
         amap f (CBuiltinTypesCompatible a_1 a_2 a_3)
           = CBuiltinTypesCompatible a_1 a_2 (f a_3)
 
-
-instance (CNode t1) => CNode (CConstant t1) where
+instance CNode t1 => CNode (CConstant t1) where
         nodeInfo (CIntConst _ n) = nodeInfo n
         nodeInfo (CCharConst _ n) = nodeInfo n
         nodeInfo (CFloatConst _ n) = nodeInfo n
         nodeInfo (CStrConst _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CConstant t1) where
+instance CNode t1 => Pos (CConstant t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CConstant where
         fmap _f (CIntConst a1 a2) = CIntConst a1 (_f a2)
         fmap _f (CCharConst a1 a2) = CCharConst a1 (_f a2)
         fmap _f (CFloatConst a1 a2) = CFloatConst a1 (_f a2)
         fmap _f (CStrConst a1 a2) = CStrConst a1 (_f a2)
 
-
 instance Annotated CConstant where
         annotation (CIntConst _ n) = n
         annotation (CCharConst _ n) = n
@@ -1352,18 +1367,14 @@ instance Annotated CConstant where
         amap f (CFloatConst a_1 a_2) = CFloatConst a_1 (f a_2)
         amap f (CStrConst a_1 a_2) = CStrConst a_1 (f a_2)
 
-
-instance (CNode t1) => CNode (CStringLiteral t1) where
+instance CNode t1 => CNode (CStringLiteral t1) where
         nodeInfo (CStrLit _ n) = nodeInfo n
-
-instance (CNode t1) => Pos (CStringLiteral t1) where
+instance CNode t1 => Pos (CStringLiteral t1) where
         posOf x = posOf (nodeInfo x)
 
-
 instance Functor CStringLiteral where
         fmap _f (CStrLit a1 a2) = CStrLit a1 (_f a2)
 
-
 instance Annotated CStringLiteral where
         annotation (CStrLit _ n) = n
         amap f (CStrLit a_1 a_2) = CStrLit a_1 (f a_2)
